"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.getOptions = void 0;

var _react = _interopRequireDefault(require("react"));

var _axios = _interopRequireDefault(require("axios"));

var _lodash = require("lodash");

var _lib = require("q3-ui-forms/lib");

var _utils = require("../utils");

var _reducer = _interopRequireDefault(require("./reducer"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getOptions = (url, key, pathToLabel) => {
  return _axios.default.get(url).then(({
    data
  }) => (0, _lodash.get)(data, key, []).map(i => ({
    label: (0, _lodash.get)(i, pathToLabel),
    value: i.id
  }))).catch(() => {
    return [];
  });
};

exports.getOptions = getOptions;

var _default = ({
  url,
  redirectOnSearch,
  key,
  pluralized,
  runOnInit = false,
  strategy = 'formik',
  decorators = {},
  location = {},
  history = {}
}) => {
  if (!url) throw new Error('Requires a valid URL');
  const {
    search
  } = location;
  const {
    onStart,
    onComplete
  } = (0, _lib.useFormHandler)(strategy);
  const {
    onSuccess,
    onFail
  } = (0, _lib.useNotification)();

  const [state, dispatch] = _react.default.useReducer(_reducer.default, {
    fetching: runOnInit,
    progress: 0
  });

  const call = (type, data = {}, err = {}) => dispatch({
    key,
    data,
    pluralized,
    err,
    type
  });

  const handleRequest = (promise, actions, verb) => {
    onStart(actions);
    return promise.then(({
      data
    }) => {
      call(verb, data);
      onComplete(null, actions);
      onSuccess((0, _lodash.get)(data, 'message'));
      return null;
    }).catch(err => {
      onComplete((0, _lodash.get)(err, 'data'), actions);
      onFail((0, _lodash.get)(err, 'data.message'));
      return err;
    });
  };

  const wrapUpdateFn = (id, verb) => (values, actions) => {
    (0, _lodash.invoke)(decorators, verb, values);
    return handleRequest((0, _lodash.invoke)(_axios.default, verb, (0, _utils.makePath)([url, id]), values), actions, _constants.UPDATED);
  };

  const methods = {
    get(query = '') {
      call(_constants.FETCHING);
      return _axios.default.get(`${url}${query}`).then(({
        data
      }) => {
        call(_constants.FETCHED, data);
        return data;
      }).catch(err => {
        call(_constants.FETCHED, null, err);
        onFail((0, _lodash.get)(err, 'message'));
        return null;
      });
    },

    remove(id) {
      return () => _axios.default.delete((0, _utils.makePath)([url, id])).then(({
        data
      }) => {
        onSuccess((0, _lodash.get)(data, 'message'));
        call(_constants.DELETED, {
          id
        });
        return null;
      }).catch(err => {
        onFail((0, _lodash.get)(err, 'message'));
        return err;
      });
    },

    patch(id) {
      const {
        name
      } = methods.patch;
      return wrapUpdateFn(id, name);
    },

    put(id) {
      const {
        name
      } = methods.put;
      return wrapUpdateFn(id, name);
    },

    post(values, actions) {
      const {
        name
      } = methods.post;
      (0, _lodash.invoke)(decorators, name, values);
      return handleRequest(_axios.default.post(url, values), actions, _constants.CREATED);
    }

  };

  _react.default.useEffect(() => {
    if (runOnInit && !redirectOnSearch) {
      methods.get(search);
    } else if (redirectOnSearch && search) {
      const {
        push
      } = history;
      push(`${redirectOnSearch}${search}`);
    }
  }, [search]);

  return { ...state,
    ...methods
  };
};

exports.default = _default;